@@ -13,6 +13,7 @@ use overload '""' => \&as_string;
 our $VERSION = '0.0.2';
 
 use JavaScript::Writer;
+use JavaScript::Writer::Block;
 
 sub new {
     my $class = shift;
@@ -32,14 +33,11 @@ sub arguments {
 sub as_string {
     my $self = shift;
     my $sub = $self->body;
-    my $function_body = sub {
-        my $js = shift;
-        $sub->($js);
-        return $js;
-    }->(JavaScript::Writer->new);
+    my $function_body = JavaScript::Writer::Block->new;
+    $function_body->body($self->body);
     my $name = $self->name ? " $self->{name}" : "";
     my $args = join(",", $self->arguments);
-    return "function${name}($args){${function_body}}";
+    return "function${name}($args)${function_body}";
 }
 
 1;
